@@ -4,21 +4,31 @@ use Moo;
 use strict;
 use warnings;
 
-our $VERSION = "0.003";
+our $VERSION = "0.004";
 $VERSION = eval $VERSION;
 
 use Carp;
 use List::Util qw (first);
-use Class::Load qw (load_class);
+use DBIx::Table::TestDataGenerator::TableProbe;
 use DBIx::Table::TestDataGenerator::DBDriverUtils;
 use DBIx::Table::TestDataGenerator::TreeUtils;
 
-has dbh => (
+has dsn => (
     is       => 'ro',
     required => 1
 );
 
-has schema => (
+has user => (
+    is       => 'ro',
+    required => 1
+);
+
+has password => (
+    is       => 'ro',
+    required => 1
+);
+
+has on_the_fly_schema_sql => (
     is       => 'ro',
     required => 0
 );
@@ -28,8 +38,9 @@ has table => (
     required => 1
 );
 
-has custom_probe_class => (
-    is       => 'ro',
+#Todo: make private!
+has probe => (
+    is       => 'rw',
     required => 0
 );
 
@@ -44,35 +55,30 @@ sub create_testdata {
     my $min_children   = $args{min_children};
     my $min_roots      = $args{min_roots};
 
-    my $tree_utils = DBIx::Table::TestDataGenerator::TreeUtils->new();
-    my $db_driver_utils =
-        DBIx::Table::TestDataGenerator::DBDriverUtils->new();
+    my $tree_utils      = DBIx::Table::TestDataGenerator::TreeUtils->new();
+    my $db_driver_utils = DBIx::Table::TestDataGenerator::DBDriverUtils->new();
 
-    $db_driver_utils->check_db_handle( $self->dbh );
-    my $database = $db_driver_utils->get_database( $self->dbh );
-
-    #determine the TableProbe subclass for the current DBMS
-    my $probe_class = $self->custom_probe_class
-        || 'DBIx::Table::TestDataGenerator::TableProbe::'
-        . $db_driver_utils->db_driver_name( $self->dbh );
-
-    load_class($probe_class);
-
-    #$probe will be used to determine (meta)data of our target table
-    my $probe = $probe_class->new(
-        dbh      => $self->dbh,
-        database => $database,
-        schema   => $self->schema,
-        table    => $self->table
+    $self->probe(
+        DBIx::Table::TestDataGenerator::TableProbe->new(
+            dsn                   => $self->dsn,
+            user                  => $self->user,
+            password              => $self->password,
+            on_the_fly_schema_sql => $self->on_the_fly_schema_sql,
+            table                 => $self->table
+        )
     );
 
+    #dump DBIC schema to file
+    $self->probe->dump_schema( );
+
+    #Todo: reimplement seed functionality
     #seed Perl and database random number generation
-    if ( defined $seed ) {
-        croak "seed should be an integer, which $seed is not"
-            unless $seed =~ /^\d+$/;
-        srand $seed;
-        $probe->seed($seed);
-    }
+    # if ( defined $seed ) {
+    # croak "seed should be an integer, which $seed is not"
+    # unless $seed =~ /^\d+$/;
+    # srand $seed;
+    # $self->probe->seed($seed);
+    # }
 
     my ( $num_records_added, $num_roots );
 
@@ -96,28 +102,28 @@ sub create_testdata {
         || defined $min_roots )
     {
         croak
-            'to handle a self-reference, you need to specify max_tree_depth, '
-            . 'min_children and min_roots, the min_roots parameter is missing'
-            unless defined $min_roots;
+          'to handle a self-reference, you need to specify max_tree_depth, '
+          . 'min_children and min_roots, the min_roots parameter is missing'
+          unless defined $min_roots;
         croak
-            'to handle a self-reference, you need to specify max_tree_depth, '
-            . 'min_children and min_roots, the min_children parameter is missing'
-            unless defined $min_children;
+          'to handle a self-reference, you need to specify max_tree_depth, '
+          . 'min_children and min_roots, the min_children parameter is missing'
+          unless defined $min_children;
         croak
-            'to handle a self-reference, you need to specify max_tree_depth, '
-            . 'min_children and min_roots, the max_tree_depth parameter is missing'
-            unless defined $max_tree_depth;
+          'to handle a self-reference, you need to specify max_tree_depth, '
+          . 'min_children and min_roots, the max_tree_depth parameter is missing'
+          unless defined $max_tree_depth;
     }
 
     #Determine whether the user has provided all informations needed to handle
     #a possible self-reference.
     my $handle_self_ref_wanted =
-           defined $max_tree_depth
-        && defined $min_children
-        && defined $min_roots;
+         defined $max_tree_depth
+      && defined $min_children
+      && defined $min_roots;
 
     #Determine original number of records in target table.
-    my $num_records_orig = $probe->num_records();
+    my $num_records_orig = $self->probe->num_records();
 
     if ( $num_records_orig == 0 ) {
         croak 'The target table ' . $self->table . ' must not be empty';
@@ -126,8 +132,8 @@ sub create_testdata {
     my $num_records_to_insert = $target_size - $num_records_orig;
     if ( $num_records_to_insert <= 0 ) {
         print 'already enough records in table '
-            . $self->table
-            . "\ncurrent number: $num_records_orig, requested: $target_size\n";
+          . $self->table
+          . "\ncurrent number: $num_records_orig, requested: $target_size\n";
         return 0;
     }
 
@@ -138,7 +144,7 @@ sub create_testdata {
     ###HANDLE COLUMNS IN UNIQUE CONSTRAINTS###
 
     #First, get information about the columns in unique constraints.
-    my %unique_cols_info = %{ $probe->unique_columns_with_max(0) };
+    my %unique_cols_info = %{ $self->probe->unique_columns_with_max(0) };
     my %unique_cols_to_incr;
 
     #Next, for each unique constraint we determine a column whose value will
@@ -148,21 +154,20 @@ sub create_testdata {
     #For the selected column, a (data type dependent) incrementor is provided
     #by the TableProbe class.
     my $type_preference_for_incrementing =
-        $probe->get_type_preference_for_incrementing();
+      $self->probe->get_type_preference_for_incrementing();
 
     for my $constraint_name ( keys %unique_cols_info ) {
         my %constraint_info = %{ $unique_cols_info{$constraint_name} };
         my $selected_data_type =
-            first { $constraint_info{$_} }
-        @{$type_preference_for_incrementing};
+          first { $constraint_info{$_} } @{$type_preference_for_incrementing};
         croak "Could not handle unique constraint $constraint_name, "
-            . "$probe_class does not know how to increment colunms of any "
-            . "of the constrained columns' data types."
-            unless defined $selected_data_type;
+          . "Don't know how to increment columns of any "
+          . "of the constrained columns' data types."
+          unless defined $selected_data_type;
         my ( $selected_unique_col, $max ) =
-            @{ @{ $constraint_info{$selected_data_type} }[0] };
+          @{ @{ $constraint_info{$selected_data_type} }[0] };
         $unique_cols_to_incr{$selected_unique_col} =
-            $probe->get_incrementor( $selected_data_type, $max );
+          $self->probe->get_incrementor( $selected_data_type, $max );
     }
 
     push @handled_columns, keys %unique_cols_to_incr;
@@ -170,7 +175,7 @@ sub create_testdata {
     ###HANDLE COLUMNS IN PRIMARY KEY CONSTRAINTS###
 
     #Determine the dictionary pkey->datatype(pkey) of the pkey columns.
-    my %pkey_cols_info = %{ $probe->unique_columns_with_max(1) };
+    my %pkey_cols_info = %{ $self->probe->unique_columns_with_max(1) };
 
     #Determine the column names in the primary key. This is needed only
     #for determining later on if there is a self-reference.
@@ -196,17 +201,17 @@ sub create_testdata {
         #Determine the pkey column to be incremented and its incrementor
         #similar logic as for unique constraint columns.
         my $selected_data_type =
-            first { $constraint_info{$_} }
-        @{$type_preference_for_incrementing};
+          first { $constraint_info{$_} } @{$type_preference_for_incrementing};
         croak "Could not handle primary key constraint $constraint_name."
-            unless defined $selected_data_type;
+          unless defined $selected_data_type;
 
         my $max;
 
         ( $pkey_col_to_incr, $max ) =
-            @{ @{ $constraint_info{$selected_data_type} }[0] };
+          @{ @{ $constraint_info{$selected_data_type} }[0] };
+        $pkey_col_to_incr = lc $pkey_col_to_incr;
         $pkey_col_incrementor =
-            $probe->get_incrementor( $selected_data_type, $max );
+          $self->probe->get_incrementor( $selected_data_type, $max );
 
         push @handled_columns, $pkey_col_to_incr;
     }
@@ -222,39 +227,40 @@ sub create_testdata {
 
     #We define dictionaries relating the corresponding columns in the target
     #table to those in the referenced tables.
-    my $fkey_tables_ref = $probe->fkey_name_to_fkey_table();
+    my $fkey_tables_ref = $self->probe->fkey_name_to_source();
 
     #skip foreign key handling if there is none
 
-    my ($all_refcol_to_col_dict, $all_refcol_lists,       $fkey_self_ref,
+    my (
+        $all_refcol_to_col_dict, $all_refcol_lists,       $fkey_self_ref,
         $parent_pkey_col,        %all_refcol_to_col_dict, %all_refcol_lists,
         $handle_self_ref,        $selfref_tree
     );
 
     if ( keys %{$fkey_tables_ref} > 0 ) {
         $all_refcol_to_col_dict =
-            $probe->fkey_referenced_cols_to_referencing_cols();
+          $self->probe->fkey_referenced_cols_to_referencing_cols();
 
         $all_refcol_lists =
-            $probe->fkey_referenced_cols( $fkey_tables_ref,
+          $self->probe->fkey_referenced_cols( $fkey_tables_ref,
             \@pkey_column_names );
 
         %all_refcol_to_col_dict = %{$all_refcol_to_col_dict};
         %all_refcol_lists       = %{$all_refcol_lists};
 
-     #If a self-reference is to be handled, define the tree of self-references
-     #which will be used to determine the parent records later on.
+       #If a self-reference is to be handled, define the tree of self-references
+       #which will be used to determine the parent records later on.
         if (   $handle_self_ref_wanted
             && defined $pkey_col_to_incr
             && @pkey_column_names == 1 )
         {
             ( $fkey_self_ref, $parent_pkey_col ) = @{
-                $probe->get_self_reference( $fkey_tables_ref,
+                $self->probe->get_self_reference( $fkey_tables_ref,
                     $pkey_column_names[0] )
             };
             if ( defined $fkey_self_ref && defined $parent_pkey_col ) {
                 $selfref_tree =
-                    $probe->selfref_tree( $pkey_col_to_incr,
+                  $self->probe->selfref_tree( $pkey_col_to_incr,
                     $parent_pkey_col );
 
                 push @handled_columns, $parent_pkey_col;
@@ -264,31 +270,27 @@ sub create_testdata {
         }
 
         for ( values %all_refcol_to_col_dict ) {
-            push @handled_columns, values $_;
+            push @handled_columns, values %{$_};
         }
     }
 
     ###HANDLE COLUMNS WHERE VALUES ARE TAKEN FROM TARGET TABLE ITSELF###
 
-    my @all_cols = @{ $probe->column_names() };
+    my @all_cols = @{ $self->probe->column_names() };
 
     #Filter out already handled columns.
     my @cols_from_target_table =
-        grep {
+      grep {
         my $c = $_;
-        !( grep { $_ eq $c } @handled_columns )
-        } @all_cols;
+        !( grep { lc $_ eq lc $c } @handled_columns )
+      } @all_cols;
 
-    my $cols_from_target_table_list = join ', ', @cols_from_target_table;
-
-    my ( %fkey_random_val_caches, @target_table_cache,
-        $pkey_col_to_incr_val );
+    my ( %fkey_random_val_caches, @target_table_cache, $pkey_col_to_incr_val );
 
     $num_records_added = 0;
 
     #Define the prepared insert statement.
-    my $sth_insert =
-        $self->dbh->prepare( $probe->insert_statement( \@all_cols ) );
+    $self->probe->prepare_insert( \@all_cols );
 
     ###MAIN LOOP: EACH STEP ADDS A NEW RECORD###
     for ( 1 .. $num_records_to_insert ) {
@@ -310,38 +312,37 @@ sub create_testdata {
             if ( $num_records_added >= $num_random ) {
                 %insert = (
                     %insert,
-                    %{  @{ $fkey_random_val_caches{$fkey} }
-                            [ int rand $num_random ]
+                    %{
+                        @{ $fkey_random_val_caches{$fkey} }
+                          [ int rand $num_random ]
                     }
                 );
             }
 
-           #...otherwise, get we get the values from randomly selected records
+            #...otherwise, get we get the values from randomly selected records
             else {
 
                 #Correspondence between columns in target table and referenced
                 #columns:
                 my %refcol_to_col_dict = %{ $all_refcol_to_col_dict{$fkey} };
 
-              #List of referenced columns:
-              #TODO: check if %all_refcol_lists can be made a simpler data
-              #structure: we dereference an array and select the first element
-              #only!!
-                my $refcol_list = @{ $all_refcol_lists->{$fkey} }[0];
+                #List of referenced columns:
+                my $refcol_list = $all_refcol_lists->{$fkey};
 
                 #If we do not handle a self-reference or the current foreign
                 #key is not the one defining it, take the values randomly
                 #from the referenced table...
                 if ( !$handle_self_ref || $fkey ne $fkey_self_ref ) {
-                    my %insert_part =
-                        %{ $probe->random_record( $fkey_table, $refcol_list )
-                        };
+                    my %insert_part = %{
+                        $self->probe->random_record( $fkey_table,
+                            $refcol_list, 1 )
+                    };
 
                     #To define our insert we need to replace the column names
                     #from the referenced table by those in the target table.
                     for my $key ( keys %insert_part ) {
                         $insert_part{ $refcol_to_col_dict{$key} } =
-                            delete $insert_part{$key};
+                          delete $insert_part{$key};
                     }
 
                     %insert = ( %insert, %insert_part );
@@ -359,7 +360,7 @@ sub create_testdata {
                     if ( $num_records_added == 0 ) {
 
                         $num_roots =
-                            $probe->num_roots( $pkey_col_to_incr,
+                          $self->probe->num_roots( $pkey_col_to_incr,
                             $parent_pkey_col );
 
                         #If we want to have a balanced tree, we want to ensure
@@ -377,9 +378,9 @@ sub create_testdata {
 
                             if ( $min_children > 1 ) {
                                 $i =
-                                    $r
-                                    * ( $min_children**$max_tree_depth
-                                        - 1 / ( $min_children - 1 ) );
+                                  $r *
+                                  ( $min_children**$max_tree_depth -
+                                      1 / ( $min_children - 1 ) );
                             }
                             else {
                                 $i = $r * $max_tree_depth;
@@ -394,12 +395,12 @@ sub create_testdata {
                     #If necessary, add a root node.
                     if ( $num_roots < $min_roots ) {
                         $selfref_tree->{$pkey_col_to_incr_val} =
-                            [$pkey_col_to_incr_val];
+                          [$pkey_col_to_incr_val];
                         $insert{$parent_pkey_col} = $pkey_col_to_incr_val;
 
                         #Store value in cache.
                         push @{ $fkey_random_val_caches{$fkey} },
-                            { $parent_pkey_col => $pkey_col_to_incr_val };
+                          { $parent_pkey_col => $pkey_col_to_incr_val };
                         $num_roots++;
                     }
                     else {
@@ -416,7 +417,7 @@ sub create_testdata {
 
                         #Store value in cache.
                         push @{ $fkey_random_val_caches{$fkey} },
-                            { $parent_pkey_col => $parent_pkey };
+                          { $parent_pkey_col => $parent_pkey };
                     }
 
                 }
@@ -437,35 +438,37 @@ sub create_testdata {
             if ( $num_records_added >= $num_random ) {
 
                 #Select values randomly from the cache.
-                %insert = (
-                    %insert, %{ $target_table_cache[ int rand $num_random ] }
-                );
+                %insert =
+                  ( %insert, %{ $target_table_cache[ int rand $num_random ] } );
             }
             else {
 
                 #Select values randomly from the target table.
-                my $values =
-                    $probe->random_record( $self->table,
-                    $cols_from_target_table_list );
-                %insert = ( %insert, %{$values} );
+                my %values = %{
+                    $self->probe->random_record( $self->table,
+                        \@cols_from_target_table )
+                };
+
+                #change all keys to lowercase
+                %values =
+                  map { lc $_ => $values{$_} } keys %values;
+                %insert = ( %insert, %values );
 
                 #Store values in cache.
-                push @target_table_cache, $values;
+                push @target_table_cache, \%values;
             }
         }
 
         #Execute the insert.
         my @val_list = map { $insert{$_} } @all_cols;
-        $sth_insert->execute(@val_list);
+        $self->probe->execute_insert( \@val_list );
 
         $num_records_added++;
     }
 
     #Commit all inserts. From DBI doc: If AutoCommit is on, then calling
     #commit will issue a "commit ineffective with AutoCommit" warning.
-    $self->dbh->commit()
-        unless $self->dbh->{AutoCommit}
-        or croak "Could not commit the inserts:\n" . $self->dbh->errstr;
+    $self->probe->commit();
 
     return $num_records_added;
 }
@@ -482,51 +485,41 @@ DBIx::Table::TestDataGenerator - Automatic test data creation, cross DBMS
 
 =head1 VERSION
 
-Version 0.0.1
+Version 0.0.4
 
 =head1 SYNOPSIS
 
-    use DBIx::Table::TestDataGenerator;
+	use DBIx::Table::TestDataGenerator;
+	
+	my $generator = DBIx::Table::TestDataGenerator->new(
+		dsn                    => $data_source_name,
+		user                   => $db_user_name,
+		password               => $db_password,          
+		table                  => $target_table_name,
+	);
 
-    my $generator = DBIx::Table::TestDataGenerator->new(
-            dbh                => $dbi_database_handle,
-            schema             => $schema_name,
-            table              => $target_table_name,
-    );
+	#simple usage:
+	$generator->create_testdata(
+		target_size            => $target_size,
+		num_random             => $num_random,            
+	);
 
-    #simple usage:
-    $generator->create_testdata(
-        target_size    => $target_size,
-        num_random     => $num_random,
-        seed           => $seed,
-    );
-
-    #extended usage handling a self-reference of the target table:
-    $generator->create_testdata(
-            target_size    => $target_size,
-            num_random     => $num_random,
-            seed           => $seed,
-            max_tree_depth => $max_tree_depth,
-            min_children   => $min_children,
-            min_roots      => $min_roots,
-    );
-
-
-    #instantiation using a custom DBMS handling class
-    my $generator = DBIx::Table::TestDataGenerator->new(
-            dbh                => $dbi_database_handle,
-            schema             => $schema_name,
-            table              => $target_table_name,
-            custom_probe_class => $custom_probe_class_name,
-    );
+	#extended usage handling a self-reference of the target table:
+	$generator->create_testdata(
+			target_size        => $target_size,
+			num_random         => $num_random,                
+			max_tree_depth     => $max_tree_depth,
+			min_children       => $min_children,
+			min_roots          => $min_roots,
+	);
 
 =head1 DESCRIPTION
 
 There is often the need to create test data in database tables, e.g. to test database client performance. The existence of constraints on a table makes it non-trivial to come up with a way to add records to it.
 
-The current module inspects the tables' constraints and adds a desired number of records. The values of the fields either come from the table itself (possibly incremented to satisfy uniqueness constraints) or from tables referenced by foreign key constraints. The choice of the copied values is random for a number of runs the user can choose, afterwards the values are chosen randomly from a cache, reducing database traffic for performance reasons. The user can define seeds for the randomization to be able to reproduce a test run. One nice thing about this way to construct new records is that at least at first sight, the added data looks like real data, at least as real as the data initially present in the table was.
+The current module inspects the tables' constraints and adds a desired number of records. The values of the fields either come from the table itself (possibly incremented to satisfy uniqueness constraints) or from tables referenced by foreign key constraints. The choice of the copied values is random for a number of runs the user can choose, afterwards the values are chosen randomly from a cache, reducing database traffic for performance reasons. One nice thing about this way to construct new records is that the additional data is similar to the data initially present in the table.
 
-A main goal of the module is to reduce configuration to the absolute minimum by automatically determining information about the target table, in particular its constraints. Another goal is to support as many DBMSs as possible. Currently Oracle, PostgreSQL and SQLite are supported, further DBMSs are in the work and one can add further databases or change the default behaviour by writing a class satisfying the role defined in DBIx::Table::TestDataGenerator::TableProbe.pm. NOTE: A major refactoring is on its way, see section FURTHER DEVELOPMENT.
+A main goal of the module is to reduce configuration to the absolute minimum by automatically determining information about the target table, in particular its constraints. Another goal is to support as many DBMSs as possible, this has been achieved by basing it on DBIx::Class modules.
 
 In the synopsis, an extended usage has been mentioned. This refers to the common case of having a self-reference on a table, i.e. a one-column wide foreign key of a table to itself where the referenced column constitutes the primary key. Such a parent-child relationship defines a rootless tree and when generating test data it may be useful to have some control over the growth of this tree. One such case is when the parent-child relation represents a navigation tree and a client application processes this structure. In this case, one would like to have a meaningful, balanced tree structure since this corresponds to real-world examples. To control tree creation the parameters max_tree_depth, min_children and min_roots are provided. Note that the nodes are being added in a depth-first manner.
 
@@ -536,40 +529,38 @@ In the synopsis, an extended usage has been mentioned. This refers to the common
 
 Arguments:
 
-=over 4
+=over 4 
 
-=item * dbh: required DBI database handle
+=item * dsn: required DBI data source name
 
-=item * schema: optional database schema name
+=item * user: required database user
+
+=item * password: required database user's password
 
 =item * table: required name of the target table
 
-=item * custom_probe_class: optional custom probe class name
-
 =back
 
 Return value:
 
-a new TestDataGenerator object
+A new TestDataGenerator object
 
-Creates a new TestDataGenerator object. If the DBMS in question does not support the concept of a schema, the corresponding argument may be omitted. If a DBMS currently not supported by DBI::Table::TestDataGenerator is to be supported, or the behaviour of the current TableProbe class responsible for handling the DBMS must be changed, one may provide the optional custom_probe_class parameter. custom_probe_class being the name of a custom class impersonating the TableProbe role.
+=head2 dsn
 
-=head2 dbh
+Accessor for the DBI data source name.
 
-Accessor for the DBI database handle.
+=head2 user
 
-=head2 schema
+Accessor for the database user.
 
-Accessor for the database schema name.
+=head2 password
+
+Accessor for the database user's password.
 
 =head2 table
 
 Accessor for the name of the target table.
 
-=head2 custom_probe_class
-
-Accessor for the name of a custom class impersonating the TableProbe role.
-
 =head2 create_testdata
 
 This is the main method, it creates and adds new records to the target table. In case one of the arguments max_tree_depth, min_children or min_roots has been provided, the other two must be provided as well.
@@ -586,10 +577,6 @@ The target number of rows to be reached.
 
 The first $num_random number of records use fresh random choices for their values taken from tables referenced by foreign key relations or the target table itself. These values are stored in a cache and re-used for the remaining (target_size - $num_random) records. Note that even for the remaining records there is some randomness since the combination of cached values coming from columns involved in different constraints is random.
 
-=item * seed
-
-This value must be an integer. In case it has been provided, the random selections done by the Perl code as well as those done by the database (where supported, e.g. not for SQLite) are seeded by this value resp. a value based on this value, e.g. PostgreSQL accepting only floating numbers between 0 and 1. This allows for reproducible test runs.
-
 =item * max_tree_depth
 
 In case of a self-reference, the maximum depth at which new records will be inserted. The minimum value for this parameter is 2.
@@ -617,20 +604,6 @@ To install this module, run the following commands:
 	./Build test
 	./Build install
 
-When installing from CPAN, the install tests look for the environment variables TDG_DSN (connection string), TDG_USER (user), TDG_PWD (password) and TDG_SCHEMA (schema) which may be used to test the installation against an existing database. If TDG_DSN is found, the install will try to use this connection string and the tests will fail if no valid database connection can be established. If TDG_DSN is not found, the installation creates an in-memory SQLite database provided for free by the DBD::SQLite module and tests against this database.
-
-=head1 DATABASE VERSIONS TESTED AGAINST
-
-=over 4
-
-=item * SQLite 3.7.14.1
-
-=item * Oracle 11g XE
-
-=item * PostgreSQL 9.2.1
-
-=back
-
 =head1 LIMITATIONS
 
 =over 4
@@ -639,7 +612,7 @@ When installing from CPAN, the install tests look for the environment variables
 
 =item * Only uniqueness and foreign key constraints are taken into account. Constraints such as check constraints, which are very diverse and database specific, are not handled (and most probably will not be).
 
-=item * Uniqueness constraints involving only columns which the DBMS specific TableProbe role handler does not know how to increment cannot be handled. Typically, all string and numeric data types are supported and the set of supported data types is defined by the list provided by the TableProbe role method get_type_preference_for_incrementing(). I am thinking about allowing date incrementation, too, it would be necessary then to at least add a configuration parameter defining what time incrementation step to use.
+=item * Uniqueness constraints involving only columns which the TableProbe class does not know how to increment cannot be handled. Typically, all string and numeric data types are supported and the set of supported data types is defined by the list provided by the TableProbe method get_type_preference_for_incrementing(). I am thinking about allowing date incrementation, too, it would be necessary then to at least add a configuration parameter defining what time incrementation step to use.
 
 =item * When calling create_testdata, max_tree_depth = 1 should be allowed, too, meaning that all new records will be root records.
 
@@ -651,14 +624,14 @@ When installing from CPAN, the install tests look for the environment variables
 
 =over 4
 
-=item * A major refactoring planned to be released with version 0.003 is in the works where I want to remove database specific handling with the help of DBIx::Class. Even if some DBMS specifics are left, this will help to support a broad range of DBMSs and the matureness of DBIx::Class will certainly help to keep the number of bugs low.
-
-=item * The current version handles uniqueness constraints by picking out a column involved in the constraint and incrementing it appropriately. While one may do something different in a custom TableProbe class than incrementing and even if the values are being incremented, the calculation of the increment may be different, one is constrained to handling the single selected column.
+=item * The current version handles uniqueness constraints by picking out a column involved in the constraint and incrementing it appropriately. This should be made customizable in future versions.
 
 =item * Support for transactions and specifying transaction sizes will be added.
 
 =item * It will be possible to get the SQL source of all generated inserts without having them executed on the database.
 
+=item * Currently one cannot specify a seed for the random selections used to define the generated records since the used class DBIx::Class::Helper::ResultSet::Random does not provide this. For reproducible tests this would be a nice feature.
+
 =back
 
 =head1 ACKNOWLEDGEMENTS
@@ -669,7 +642,7 @@ When installing from CPAN, the install tests look for the environment variables
 
 A big thank you to all perl coders on the dbi-dev, DBIx-Class and perl-modules mailing lists and on PerlMonks who have patiently answered my questions and offered solutions, advice and encouragement, the Perl community is really outstanding.
 
-Special thanks go to Tim Bunce (module name / advice on keeping the module extensible), Jonathan Leffler (module naming discussion / relation to existing modules / multiple suggestions for features), brian d foy (module naming discussion / mailing lists / encouragement) and the following Perl monks (see the threads for user jds17 for details): chromatic, erix,  technojosh, kejohm, Khen1950fx, salva, tobyink (3 of 4 discussion threads!), Your Mother.
+Special thanks go to Tim Bunce (module name / advice on keeping the module extensible), Jonathan Leffler (module naming discussion / relation to existing modules / multiple suggestions for features), brian d foy (module naming discussion / mailing lists / encouragement) and the following Perl monks (see the threads for user jds17 for details): chromatic, erix, technojosh, kejohm, Khen1950fx, salva, tobyink (3 of 4 discussion threads!), Your Mother.
 
 =item * Version 0.002:
 
